@@ -32,7 +32,8 @@ namespace PS2
 	enum
 	{
 		GS_NTSC_HSYNC_FREQ = 15734,
-		GS_PAL_HSYNC_FREQ = 15625
+		GS_PAL_HSYNC_FREQ = 15625,
+		GS_HDTV_480P_HSYNC_FREQ = 31469,
 	};
 
 	enum
