@@ -4,7 +4,7 @@
 #include "AppConfig.h"
 #include "StdStreamUtils.h"
 #include "../Ps2Const.h"
-#include "../Log.h"
+#include "Log.h"
 #include "../states/MemoryStateFile.h"
 #include "../iop/IopBios.h"
 #include "Vif.h"
@@ -62,7 +62,14 @@ CSubSystem::CSubSystem(uint8* iopRam, CIopBios& iopBios)
 	m_vpu1 = std::make_shared<CVpu>(1, CVpu::VPUINIT(m_microMem1, m_vuMem1, &m_VU1), m_gif, m_intc, m_ram, m_spr);
 
 	//Setup link between EE's VU context and VU0's VU context
-	m_vu0StateChangedConnection = m_vpu0->VuStateChanged.Connect([this](bool running) { Vu0StateChanged(running); });
+	m_vu0StateChangedConnection = m_vpu0->VuStateChanged.Connect([this](CVpu::VU_STATE newState) { Vu0StateChanged(newState); });
+
+	m_vu1InterruptTriggeredConnection = m_vpu1->VuInterruptTriggered.Connect(
+	    [this]() {
+		    uint32 currentState = m_intc.GetRegister(CINTC::INTC_STAT);
+		    assert((currentState & (1 << CINTC::INTC_LINE_VU1)) == 0);
+		    m_intc.AssertLine(CINTC::INTC_LINE_VU1);
+	    });
 
 	//EmotionEngine context setup
 	{
@@ -234,7 +241,7 @@ int CSubSystem::ExecuteCpu(int quota)
 	int executed = 0;
 	if(m_EE.m_State.callMsEnabled)
 	{
-		if(!m_vpu0->IsVuRunning())
+		if(m_vpu0->IsVuReady())
 		{
 			//callMs mode over
 			m_EE.m_State.callMsAddr = m_VU0.m_State.nPC;
@@ -255,12 +262,11 @@ int CSubSystem::ExecuteCpu(int quota)
 		case MIPS_EXCEPTION_TLB:
 			m_os->HandleTLBException();
 			break;
-		case MIPS_EXCEPTION_CALLMS:
+		case MIPS_EXCEPTION_VU_CALLMS:
 			assert(m_EE.m_State.callMsEnabled);
 			if(m_EE.m_State.callMsEnabled)
 			{
 				//We are in callMs mode
-				assert(!m_vpu0->IsVuRunning());
 				m_vpu0->ExecuteMicroProgram(m_EE.m_State.callMsAddr);
 				m_EE.m_State.nHasException = MIPS_EXCEPTION_NONE;
 			}
@@ -300,11 +306,11 @@ bool CSubSystem::IsCpuIdle() const
 
 void CSubSystem::CountTicks(int ticks)
 {
-	if(!m_vpu0->IsVuRunning() || (m_vpu0->IsVuRunning() && !m_vpu0->GetVif().IsWaitingForProgramEnd()))
+	if(m_vpu0->IsVuReady() || (m_vpu0->IsVuRunning() && !m_vpu0->GetVif().IsWaitingForProgramEnd()))
 	{
 		m_dmac.ResumeDMA0();
 	}
-	if(!m_vpu1->IsVuRunning() || (m_vpu1->IsVuRunning() && !m_vpu1->GetVif().IsWaitingForProgramEnd()))
+	if(m_vpu1->IsVuReady() || (m_vpu1->IsVuRunning() && !m_vpu1->GetVif().IsWaitingForProgramEnd()))
 	{
 		m_dmac.ResumeDMA1();
 	}
@@ -454,6 +460,11 @@ uint32 CSubSystem::IOPortReadHandler(uint32 nAddress)
 	{
 		nReturn = m_dmac.GetRegister(nAddress);
 	}
+	else if(nAddress >= CVpu::EE_ADDR_VU1AREA_START && nAddress <= CVpu::EE_ADDR_VU1AREA_END)
+	{
+		uint32 offset = nAddress - CVpu::EE_ADDR_VU1AREA_START;
+		nReturn = HandleVu1AreaRead(offset);
+	}
 	else if(nAddress >= 0x12000000 && nAddress <= 0x1200108C)
 	{
 		if(m_gs != NULL)
@@ -558,10 +569,14 @@ uint32 CSubSystem::IOPortWriteHandler(uint32 nAddress, uint32 nData)
 		uint32 offset = nAddress - CVpu::EE_ADDR_VU1AREA_START;
 		HandleVu1AreaWrite(offset, nData);
 	}
+	else if(nAddress == CVpu::EE_ADDR_VU_FBRST)
+	{
+		m_vpu1->SetFbrst((nData >> 8) & 0xF);
+	}
 	else if(nAddress == CVpu::EE_ADDR_VU_CMSAR1)
 	{
 		bool validAddress = (nData & 0x7) == 0;
-		if(!m_vpu1->IsVuRunning() && validAddress)
+		if(validAddress)
 		{
 			m_vpu1->ExecuteMicroProgram(nData);
 		}
@@ -626,7 +641,7 @@ uint32 CSubSystem::Vu0IoPortWriteHandler(uint32 address, uint32 value)
 	return 0;
 }
 
-void CSubSystem::Vu0StateChanged(bool running)
+void CSubSystem::Vu0StateChanged(CVpu::VU_STATE newState)
 {
 	auto flushPipelines =
 	    [](CMIPS& ctx) {
@@ -638,7 +653,8 @@ void CSubSystem::Vu0StateChanged(bool running)
 		    VUShared::CheckFlagPipelineImmediate(VUShared::g_pipeInfoClip, &ctx, VUShared::LATENCY_MAC);
 	    };
 
-	if(running)
+	assert(newState != CVpu::VU_STATE_STOPPED);
+	if(newState == CVpu::VU_STATE_RUNNING)
 	{
 		flushPipelines(m_EE);
 		CopyVuState(m_VU0, m_EE);
@@ -712,6 +728,27 @@ void CSubSystem::CopyVuState(CMIPS& dst, const CMIPS& src)
 	dst.m_State.pipeClip.index = src.m_State.pipeClip.index;
 }
 
+uint32 CSubSystem::HandleVu1AreaRead(uint32 offset)
+{
+	assert(!m_vpu1->IsVuRunning());
+	assert(offset < 0x400);
+	uint32 result = 0;
+	switch(offset)
+	{
+	case 0x3A0:
+		result = m_vpu1->GetContext().m_State.nPC;
+		break;
+	case 0x3A4:
+	case 0x3A8:
+	case 0x3AC:
+		break;
+	default:
+		assert(false);
+		break;
+	}
+	return result;
+}
+
 void CSubSystem::HandleVu1AreaWrite(uint32 offset, uint32 value)
 {
 	assert(!m_vpu1->IsVuRunning());
