@@ -54,10 +54,9 @@ void CMA_VU::CLower::CompileInstruction(uint32 address, CMipsJitter* codeGen, CM
 	}
 }
 
-void CMA_VU::CLower::SetRelativePipeTime(uint32 relativePipeTime, uint32 compileHints)
+void CMA_VU::CLower::SetRelativePipeTime(uint32 relativePipeTime)
 {
 	m_relativePipeTime = relativePipeTime;
-	m_compileHints = compileHints;
 }
 
 void CMA_VU::CLower::SetBranchAddress(bool nCondition, int32 nOffset)
