@@ -176,6 +176,34 @@ void VUShared::PullVector(CMipsJitter* codeGen, uint8 dest, size_t vector)
 	                    DestinationHasElement(dest, 3));
 }
 
+void VUShared::PushBcElement(CMipsJitter* codeGen, size_t offset)
+{
+	if(
+	    (offset == offsetof(CMIPS, m_State.nCOP2[0].nV0)) ||
+	    (offset == offsetof(CMIPS, m_State.nCOP2[0].nV1)) ||
+	    (offset == offsetof(CMIPS, m_State.nCOP2[0].nV2)))
+	{
+		codeGen->MD_PushCstExpandS(0.0f);
+	}
+	else if(offset == offsetof(CMIPS, m_State.nCOP2[0].nV3))
+	{
+		codeGen->MD_PushCstExpandS(1.0f);
+	}
+	else if(
+	    (offset >= offsetof(CMIPS, m_State.nCOP2[1].nV0)) &&
+	    (offset <= offsetof(CMIPS, m_State.nCOP2[31].nV3)))
+	{
+		size_t vectorOffset = offset & ~(sizeof(uint128) - 1);
+		size_t vectorElem = (offset - offsetof(CMIPS, m_State.nCOP2[0].nV0)) % sizeof(uint128);
+		codeGen->MD_PushRelElementExpandW(vectorOffset, vectorElem / 4);
+	}
+	else
+	{
+		codeGen->PushRel(offset);
+		codeGen->MD_ExpandW();
+	}
+}
+
 void VUShared::PushIntegerRegister(CMipsJitter* codeGen, unsigned int regIdx)
 {
 	regIdx &= 0xF;
@@ -196,6 +224,43 @@ void VUShared::PullIntegerRegister(CMipsJitter* codeGen, unsigned int regIdx)
 	codeGen->PullRel(offsetof(CMIPS, m_State.nCOP2VI[regIdx]));
 }
 
+void VUShared::MakeComparableFromFloat(CMipsJitter* codeGen)
+{
+	uint32 valueCursor = codeGen->GetTopCursor();
+
+	//Compute mask
+	codeGen->PushCursor(valueCursor);
+	codeGen->MD_SraW(31);
+	uint32 maskCursor = codeGen->GetTopCursor();
+
+	//Make neg
+	codeGen->MD_PushCstExpandW(0xFFFFFFFFU);
+	codeGen->PushCursor(valueCursor);
+	codeGen->MD_SubW();
+	codeGen->PushCursor(maskCursor);
+	codeGen->MD_And();
+
+	//Make pos
+	codeGen->MD_PushCstExpandW(0x7FFFFFFFU);
+	codeGen->PushCursor(valueCursor);
+	codeGen->MD_AddW();
+	codeGen->PushCursor(maskCursor);
+	codeGen->MD_Not();
+	codeGen->MD_And();
+
+	codeGen->MD_Or();
+
+	//Make it signed comparable
+	codeGen->MD_PushCstExpandW(0x80000000U);
+	codeGen->MD_SubW();
+
+	codeGen->Swap();
+	codeGen->PullTop();
+
+	codeGen->Swap();
+	codeGen->PullTop();
+}
+
 void VUShared::TestSZFlags(CMipsJitter* codeGen, uint8 dest, size_t regOffset, uint32 relativePipeTime, uint32 compileHints)
 {
 	codeGen->MD_PushRel(regOffset);
@@ -222,16 +287,14 @@ void VUShared::TestSZFlags(CMipsJitter* codeGen, uint8 dest, size_t regOffset, u
 			codeGen->PushCst(FLAG_PIPELINE_SLOTS - 1);
 			codeGen->And();
 
-			codeGen->Shl(2);
-			codeGen->AddRef();
-			codeGen->LoadFromRef();
+			codeGen->LoadFromRefIdx();
 		}
 
 		codeGen->Or();
 		QueueInFlagPipeline(g_pipeInfoSticky, codeGen, LATENCY_MAC, relativePipeTime);
 	}
 
-	if((compileHints & COMPILEHINT_SKIPFMACUPDATE) == 0)
+	if((compileHints & COMPILEHINT_SKIP_FMAC_UPDATE) == 0)
 	{
 		QueueInFlagPipeline(g_pipeInfoMac, codeGen, LATENCY_MAC, relativePipeTime);
 	}
@@ -367,7 +430,7 @@ void VUShared::ADD_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 	codeGen->MD_ClampS();
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
@@ -383,7 +446,7 @@ void VUShared::ADDA_base(CMipsJitter* codeGen, uint8 dest, size_t fs, size_t ft,
 	codeGen->MD_PushRel(fs);
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
@@ -402,7 +465,7 @@ void VUShared::MADD_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 	codeGen->MD_ClampS();
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 		codeGen->MD_ClampS(); //Fatal Frame 1's door-blocking bug can be fixed by this
 	}
 	else
@@ -423,12 +486,13 @@ void VUShared::MADDA_base(CMipsJitter* codeGen, uint8 dest, size_t fs, size_t ft
 	codeGen->MD_ClampS();
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
 		codeGen->MD_PushRel(ft);
 	}
+	codeGen->MD_ClampS();
 	codeGen->MD_MulS();
 	codeGen->MD_AddS();
 	PullVector(codeGen, dest, offsetof(CMIPS, m_State.nCOP2A));
@@ -438,14 +502,16 @@ void VUShared::MADDA_base(CMipsJitter* codeGen, uint8 dest, size_t fs, size_t ft
 void VUShared::SUB_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs, size_t ft, bool expand, uint32 relativePipeTime, uint32 compileHints)
 {
 	codeGen->MD_PushRel(fs);
+	codeGen->MD_ClampS();
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
 		codeGen->MD_PushRel(ft);
 	}
+	codeGen->MD_ClampS();
 	codeGen->MD_SubS();
 	PullVector(codeGen, dest, fd);
 	TestSZFlags(codeGen, dest, fd, relativePipeTime, compileHints);
@@ -456,7 +522,7 @@ void VUShared::SUBA_base(CMipsJitter* codeGen, uint8 dest, size_t fs, size_t ft,
 	codeGen->MD_PushRel(fs);
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
@@ -473,7 +539,7 @@ void VUShared::MSUB_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 	codeGen->MD_PushRel(fs);
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
@@ -491,7 +557,7 @@ void VUShared::MSUBA_base(CMipsJitter* codeGen, uint8 dest, size_t fs, size_t ft
 	codeGen->MD_PushRel(fs);
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
@@ -509,7 +575,7 @@ void VUShared::MUL_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 	codeGen->MD_ClampS();
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
@@ -527,7 +593,7 @@ void VUShared::MULA_base(CMipsJitter* codeGen, uint8 dest, size_t fs, size_t ft,
 	codeGen->MD_ClampS();
 	if(expand)
 	{
-		codeGen->MD_PushRelExpand(ft);
+		PushBcElement(codeGen, ft);
 	}
 	else
 	{
@@ -543,7 +609,7 @@ void VUShared::MINI_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 	const auto pushFt = [&]() {
 		if(expand)
 		{
-			codeGen->MD_PushRelExpand(ft);
+			PushBcElement(codeGen, ft);
 		}
 		else
 		{
@@ -551,12 +617,13 @@ void VUShared::MINI_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 		}
 	};
 
-	codeGen->MD_PushRel(fs);
-	codeGen->MD_ClampS();
 	pushFt();
-	codeGen->MD_ClampS();
+	MakeComparableFromFloat(codeGen);
+	codeGen->MD_PushRel(fs);
+	MakeComparableFromFloat(codeGen);
 
-	codeGen->MD_CmpLtS();
+	//Since we don't have a less-than operator, operands are reversed (ft > fs instead of fs < ft)
+	codeGen->MD_CmpGtW();
 	auto cmp = codeGen->GetTopCursor();
 
 	//Mask FT
@@ -581,7 +648,7 @@ void VUShared::MAX_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 	const auto pushFt = [&]() {
 		if(expand)
 		{
-			codeGen->MD_PushRelExpand(ft);
+			PushBcElement(codeGen, ft);
 		}
 		else
 		{
@@ -590,11 +657,11 @@ void VUShared::MAX_base(CMipsJitter* codeGen, uint8 dest, size_t fd, size_t fs,
 	};
 
 	codeGen->MD_PushRel(fs);
-	codeGen->MD_ClampS();
+	MakeComparableFromFloat(codeGen);
 	pushFt();
-	codeGen->MD_ClampS();
+	MakeComparableFromFloat(codeGen);
 
-	codeGen->MD_CmpGtS();
+	codeGen->MD_CmpGtW();
 	auto cmp = codeGen->GetTopCursor();
 
 	//Mask FT
@@ -665,22 +732,27 @@ void VUShared::ADDi(CMipsJitter* codeGen, uint8 nDest, uint8 nFd, uint8 nFs, uin
 	//On JavaScript, using it doesn't seem to help Tri-Ace games
 	//there's probably some other rounding issue on that platform
 #if !defined(__EMSCRIPTEN__)
-	for(unsigned int i = 0; i < 4; i++)
+	if(compileHints & COMPILEHINT_USE_ACCURATE_ADDI)
 	{
-		if(!VUShared::DestinationHasElement(nDest, i)) continue;
+		for(unsigned int i = 0; i < 4; i++)
+		{
+			if(!VUShared::DestinationHasElement(nDest, i)) continue;
 
-		codeGen->PushRel(offsetof(CMIPS, m_State.nCOP2[nFs].nV[i]));
-		codeGen->PushRel(offsetof(CMIPS, m_State.nCOP2I));
-		codeGen->Call(reinterpret_cast<void*>(&FpAddTruncate), 2, Jitter::CJitter::RETURN_VALUE_32);
-		codeGen->PullRel(offsetof(CMIPS, m_State.nCOP2[nFd].nV[i]));
+			codeGen->PushRel(offsetof(CMIPS, m_State.nCOP2[nFs].nV[i]));
+			codeGen->PushRel(offsetof(CMIPS, m_State.nCOP2I));
+			codeGen->Call(reinterpret_cast<void*>(&FpAddTruncate), 2, Jitter::CJitter::RETURN_VALUE_32);
+			codeGen->PullRel(offsetof(CMIPS, m_State.nCOP2[nFd].nV[i]));
+		}
 	}
-#else
-	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[nFs]));
-	codeGen->MD_PushRelExpand(offsetof(CMIPS, m_State.nCOP2I));
-	codeGen->MD_AddS();
-	PullVector(codeGen, nDest, offsetof(CMIPS, m_State.nCOP2[nFd]));
+	else
 #endif
-
+	{
+		codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[nFs]));
+		codeGen->PushRel(offsetof(CMIPS, m_State.nCOP2I));
+		codeGen->MD_ExpandW();
+		codeGen->MD_AddS();
+		PullVector(codeGen, nDest, offsetof(CMIPS, m_State.nCOP2[nFd]));
+	}
 	TestSZFlags(codeGen, nDest, offsetof(CMIPS, m_State.nCOP2[nFd]), relativePipeTime, compileHints);
 }
 
@@ -727,8 +799,6 @@ void VUShared::ADDAq(CMipsJitter* codeGen, uint8 dest, uint8 fs, uint32 relative
 
 void VUShared::CLIP(CMipsJitter* codeGen, uint8 nFs, uint8 nFt, uint32 relativePipeTime)
 {
-	size_t tempOffset = offsetof(CMIPS, m_State.nCOP2T);
-
 	//Load previous value
 	{
 		codeGen->PushRelAddrRef(offsetof(CMIPS, m_State.pipeClip.values));
@@ -739,54 +809,50 @@ void VUShared::CLIP(CMipsJitter* codeGen, uint8 nFs, uint8 nFt, uint32 relativeP
 		codeGen->PushCst(FLAG_PIPELINE_SLOTS - 1);
 		codeGen->And();
 
-		codeGen->Shl(2);
-		codeGen->AddRef();
-		codeGen->LoadFromRef();
-		codeGen->PullRel(tempOffset);
+		codeGen->LoadFromRefIdx();
+
+		//Create some space for the new test results
+		codeGen->Shl(6);
 	}
 
-	//Create some space for the new test results
-	codeGen->PushRel(tempOffset);
-	codeGen->Shl(6);
-	codeGen->PullRel(tempOffset);
-
-	for(unsigned int i = 0; i < 3; i++)
+	//Compute test result (if both fs and ft are 0, result is 0)
+	if((nFs == 0) && (nFt == 0))
 	{
-		//c > +|w|
-		codeGen->FP_PushSingle(offsetof(CMIPS, m_State.nCOP2[nFs].nV[i]));
-		codeGen->FP_PushSingle(offsetof(CMIPS, m_State.nCOP2[nFt].nV[3]));
-		codeGen->FP_Abs();
-
-		codeGen->FP_Cmp(Jitter::CONDITION_AB);
 		codeGen->PushCst(0);
-		codeGen->BeginIf(Jitter::CONDITION_NE);
-		{
-			codeGen->PushRel(tempOffset);
-			codeGen->PushCst(1 << ((i * 2) + 0));
-			codeGen->Or();
-			codeGen->PullRel(tempOffset);
-		}
-		codeGen->EndIf();
+	}
+	else
+	{
+		codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[nFs]));
 
-		//c < -|w|
-		codeGen->FP_PushSingle(offsetof(CMIPS, m_State.nCOP2[nFs].nV[i]));
-		codeGen->FP_PushSingle(offsetof(CMIPS, m_State.nCOP2[nFt].nV[3]));
-		codeGen->FP_Abs();
-		codeGen->FP_Neg();
-
-		codeGen->FP_Cmp(Jitter::CONDITION_BL);
-		codeGen->PushCst(0);
-		codeGen->BeginIf(Jitter::CONDITION_NE);
+		//Upper bound
+		if(nFt == 0)
 		{
-			codeGen->PushRel(tempOffset);
-			codeGen->PushCst(1 << ((i * 2) + 1));
-			codeGen->Or();
-			codeGen->PullRel(tempOffset);
+			codeGen->MD_PushCstExpandS(1.0f);
 		}
-		codeGen->EndIf();
+		else
+		{
+			PushBcElement(codeGen, offsetof(CMIPS, m_State.nCOP2[nFt].nV[3]));
+			codeGen->MD_AbsS();
+		}
+
+		//Lower bound
+		if(nFt == 0)
+		{
+			codeGen->MD_PushCstExpandS(-1.0f);
+		}
+		else
+		{
+			PushBcElement(codeGen, offsetof(CMIPS, m_State.nCOP2[nFt].nV[3]));
+			codeGen->MD_AbsS();
+			codeGen->MD_NegS();
+		}
+
+		codeGen->MD_MakeClip();
 	}
 
-	codeGen->PushRel(tempOffset);
+	//Combine with previous value
+	codeGen->Or();
+
 	QueueInFlagPipeline(g_pipeInfoClip, codeGen, LATENCY_MAC, relativePipeTime);
 }
 
@@ -809,10 +875,10 @@ void VUShared::DIV(CMipsJitter* codeGen, uint8 nFs, uint8 nFsf, uint8 nFt, uint8
 	}
 	codeGen->Else();
 	{
-		codeGen->FP_PushSingle(GetVectorElement(nFs, nFsf));
-		codeGen->FP_PushSingle(GetVectorElement(nFt, nFtf));
-		codeGen->FP_Div();
-		codeGen->FP_PullSingle(destination);
+		codeGen->FP_PushRel32(GetVectorElement(nFs, nFsf));
+		codeGen->FP_PushRel32(GetVectorElement(nFt, nFtf));
+		codeGen->FP_DivS();
+		codeGen->FP_PullRel32(destination);
 
 		codeGen->PushCst(0);
 		codeGen->PullRel(offsetof(CMIPS, m_State.nCOP2DF));
@@ -825,7 +891,7 @@ void VUShared::FTOI0(CMipsJitter* codeGen, uint8 nDest, uint8 nFt, uint8 nFs)
 	if(nFt == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[nFs]));
-	codeGen->MD_ToWordTruncate();
+	codeGen->MD_ToInt32TruncateS();
 	PullVector(codeGen, nDest, offsetof(CMIPS, m_State.nCOP2[nFt]));
 }
 
@@ -834,9 +900,9 @@ void VUShared::FTOI4(CMipsJitter* codeGen, uint8 nDest, uint8 nFt, uint8 nFs)
 	if(nFt == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[nFs]));
-	codeGen->MD_PushCstExpand(16.0f);
+	codeGen->MD_PushCstExpandS(16.0f);
 	codeGen->MD_MulS();
-	codeGen->MD_ToWordTruncate();
+	codeGen->MD_ToInt32TruncateS();
 	PullVector(codeGen, nDest, offsetof(CMIPS, m_State.nCOP2[nFt]));
 }
 
@@ -845,9 +911,9 @@ void VUShared::FTOI12(CMipsJitter* codeGen, uint8 nDest, uint8 nFt, uint8 nFs)
 	if(nFt == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[nFs]));
-	codeGen->MD_PushCstExpand(4096.0f);
+	codeGen->MD_PushCstExpandS(4096.0f);
 	codeGen->MD_MulS();
-	codeGen->MD_ToWordTruncate();
+	codeGen->MD_ToInt32TruncateS();
 	PullVector(codeGen, nDest, offsetof(CMIPS, m_State.nCOP2[nFt]));
 }
 
@@ -856,9 +922,9 @@ void VUShared::FTOI15(CMipsJitter* codeGen, uint8 nDest, uint8 nFt, uint8 nFs)
 	if(nFt == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[nFs]));
-	codeGen->MD_PushCstExpand(32768.0f);
+	codeGen->MD_PushCstExpandS(32768.0f);
 	codeGen->MD_MulS();
-	codeGen->MD_ToWordTruncate();
+	codeGen->MD_ToInt32TruncateS();
 	PullVector(codeGen, nDest, offsetof(CMIPS, m_State.nCOP2[nFt]));
 }
 
@@ -935,7 +1001,7 @@ void VUShared::ITOF0(CMipsJitter* codeGen, uint8 dest, uint8 ft, uint8 fs)
 	if(ft == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[fs]));
-	codeGen->MD_ToSingle();
+	codeGen->MD_ToSingleI32();
 	PullVector(codeGen, dest, offsetof(CMIPS, m_State.nCOP2[ft]));
 }
 
@@ -944,8 +1010,8 @@ void VUShared::ITOF4(CMipsJitter* codeGen, uint8 dest, uint8 ft, uint8 fs)
 	if(ft == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[fs]));
-	codeGen->MD_ToSingle();
-	codeGen->MD_PushCstExpand(16.0f);
+	codeGen->MD_ToSingleI32();
+	codeGen->MD_PushCstExpandS(16.0f);
 	codeGen->MD_DivS();
 	PullVector(codeGen, dest, offsetof(CMIPS, m_State.nCOP2[ft]));
 }
@@ -955,8 +1021,8 @@ void VUShared::ITOF12(CMipsJitter* codeGen, uint8 dest, uint8 ft, uint8 fs)
 	if(ft == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[fs]));
-	codeGen->MD_ToSingle();
-	codeGen->MD_PushCstExpand(4096.0f);
+	codeGen->MD_ToSingleI32();
+	codeGen->MD_PushCstExpandS(4096.0f);
 	codeGen->MD_DivS();
 	PullVector(codeGen, dest, offsetof(CMIPS, m_State.nCOP2[ft]));
 }
@@ -966,8 +1032,8 @@ void VUShared::ITOF15(CMipsJitter* codeGen, uint8 dest, uint8 ft, uint8 fs)
 	if(ft == 0) return;
 
 	codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[fs]));
-	codeGen->MD_ToSingle();
-	codeGen->MD_PushCstExpand(32768.0f);
+	codeGen->MD_ToSingleI32();
+	codeGen->MD_PushCstExpandS(32768.0f);
 	codeGen->MD_DivS();
 	PullVector(codeGen, dest, offsetof(CMIPS, m_State.nCOP2[ft]));
 }
@@ -1022,36 +1088,12 @@ void VUShared::LQbase(CMipsJitter* codeGen, uint8 dest, uint8 it)
 	}
 	else
 	{
-		codeGen->AddRef();
-
-		//Try some aligned 64-bit reads first
-		for(unsigned int i = 0; i < 3; i += 2)
-		{
-			uint8 mask = (0x03 << i);
-			if((dest & mask) == mask)
-			{
-				unsigned int element = (i ^ 0x03) - 1;
-				codeGen->PushTop();
-				codeGen->PushCst(element * 4);
-				codeGen->Load64FromRefIdx(1);
-				codeGen->PullRel64(offsetof(CMIPS, m_State.nCOP2[it].nV[element]));
-				dest &= ~mask;
-			}
-		}
-
-		//Read any remaining data
-		for(unsigned int i = 0; i < 4; i++)
-		{
-			if(VUShared::DestinationHasElement(static_cast<uint8>(dest), i))
-			{
-				codeGen->PushTop();
-				codeGen->PushCst(i * 4);
-				codeGen->LoadFromRefIdx(1);
-				codeGen->PullRel(offsetof(CMIPS, m_State.nCOP2[it].nV[i]));
-			}
-		}
-
-		codeGen->PullTop();
+		codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[it]));
+		codeGen->MD_LoadFromRefIdxMasked(DestinationHasElement(dest, 0),
+		                                 DestinationHasElement(dest, 1),
+		                                 DestinationHasElement(dest, 2),
+		                                 DestinationHasElement(dest, 3));
+		codeGen->MD_PullRel(offsetof(CMIPS, m_State.nCOP2[it]));
 	}
 }
 
@@ -1071,13 +1113,29 @@ void VUShared::LQD(CMipsJitter* codeGen, uint8 dest, uint8 it, uint8 is, uint32
 	VUShared::LQbase(codeGen, dest, it);
 }
 
-void VUShared::LQI(CMipsJitter* codeGen, uint8 dest, uint8 it, uint8 is, uint32 addressMask)
+void VUShared::LQI(CMipsJitter* codeGen, uint8 dest, uint8 it, uint8 is, uint32 addressMask, const Vu1AreaAccessEmitter& vu1AreaAccessEmitter)
 {
+	if(vu1AreaAccessEmitter)
+	{
+		codeGen->PushRel(offsetof(CMIPS, m_State.nCOP2VI[is]));
+		codeGen->PushCst(CVpu::VU_ADDR_VU1AREA_START >> 4);
+		codeGen->BeginIf(Jitter::CONDITION_AE);
+		{
+			vu1AreaAccessEmitter(codeGen, is, it);
+		}
+		codeGen->Else();
+	}
+
 	codeGen->PushRelRef(offsetof(CMIPS, m_vuMem));
 	VUShared::ComputeMemAccessAddr(codeGen, is, 0, 0, addressMask);
 
 	VUShared::LQbase(codeGen, dest, it);
 
+	if(vu1AreaAccessEmitter)
+	{
+		codeGen->EndIf();
+	}
+
 	if((is & 0xF) != 0)
 	{
 		PushIntegerRegister(codeGen, is);
@@ -1413,22 +1471,22 @@ void VUShared::MULAq(CMipsJitter* codeGen, uint8 dest, uint8 fs, uint32 relative
 void VUShared::OPMULA(CMipsJitter* codeGen, uint8 nFs, uint8 nFt)
 {
 	//ACCx
-	codeGen->FP_PushSingle(GetVectorElement(nFs, VECTOR_COMPY));
-	codeGen->FP_PushSingle(GetVectorElement(nFt, VECTOR_COMPZ));
-	codeGen->FP_Mul();
-	codeGen->FP_PullSingle(GetAccumulatorElement(VECTOR_COMPX));
+	codeGen->FP_PushRel32(GetVectorElement(nFs, VECTOR_COMPY));
+	codeGen->FP_PushRel32(GetVectorElement(nFt, VECTOR_COMPZ));
+	codeGen->FP_MulS();
+	codeGen->FP_PullRel32(GetAccumulatorElement(VECTOR_COMPX));
 
 	//ACCy
-	codeGen->FP_PushSingle(GetVectorElement(nFs, VECTOR_COMPZ));
-	codeGen->FP_PushSingle(GetVectorElement(nFt, VECTOR_COMPX));
-	codeGen->FP_Mul();
-	codeGen->FP_PullSingle(GetAccumulatorElement(VECTOR_COMPY));
+	codeGen->FP_PushRel32(GetVectorElement(nFs, VECTOR_COMPZ));
+	codeGen->FP_PushRel32(GetVectorElement(nFt, VECTOR_COMPX));
+	codeGen->FP_MulS();
+	codeGen->FP_PullRel32(GetAccumulatorElement(VECTOR_COMPY));
 
 	//ACCz
-	codeGen->FP_PushSingle(GetVectorElement(nFs, VECTOR_COMPX));
-	codeGen->FP_PushSingle(GetVectorElement(nFt, VECTOR_COMPY));
-	codeGen->FP_Mul();
-	codeGen->FP_PullSingle(GetAccumulatorElement(VECTOR_COMPZ));
+	codeGen->FP_PushRel32(GetVectorElement(nFs, VECTOR_COMPX));
+	codeGen->FP_PushRel32(GetVectorElement(nFt, VECTOR_COMPY));
+	codeGen->FP_MulS();
+	codeGen->FP_PullRel32(GetAccumulatorElement(VECTOR_COMPZ));
 }
 
 void VUShared::OPMSUB(CMipsJitter* codeGen, uint8 fd, uint8 fs, uint8 ft, uint32 relativePipeTime, uint32 compileHints)
@@ -1438,28 +1496,28 @@ void VUShared::OPMSUB(CMipsJitter* codeGen, uint8 fd, uint8 fs, uint8 ft, uint32
 	uint32 dest = 0x0E;
 
 	//X
-	codeGen->FP_PushSingle(GetAccumulatorElement(VECTOR_COMPX));
-	codeGen->FP_PushSingle(GetVectorElement(fs, VECTOR_COMPY));
-	codeGen->FP_PushSingle(GetVectorElement(ft, VECTOR_COMPZ));
-	codeGen->FP_Mul();
-	codeGen->FP_Sub();
-	codeGen->FP_PullSingle(GetVectorElement(tempRegIndex, VECTOR_COMPX));
+	codeGen->FP_PushRel32(GetAccumulatorElement(VECTOR_COMPX));
+	codeGen->FP_PushRel32(GetVectorElement(fs, VECTOR_COMPY));
+	codeGen->FP_PushRel32(GetVectorElement(ft, VECTOR_COMPZ));
+	codeGen->FP_MulS();
+	codeGen->FP_SubS();
+	codeGen->FP_PullRel32(GetVectorElement(tempRegIndex, VECTOR_COMPX));
 
 	//Y
-	codeGen->FP_PushSingle(GetAccumulatorElement(VECTOR_COMPY));
-	codeGen->FP_PushSingle(GetVectorElement(fs, VECTOR_COMPZ));
-	codeGen->FP_PushSingle(GetVectorElement(ft, VECTOR_COMPX));
-	codeGen->FP_Mul();
-	codeGen->FP_Sub();
-	codeGen->FP_PullSingle(GetVectorElement(tempRegIndex, VECTOR_COMPY));
+	codeGen->FP_PushRel32(GetAccumulatorElement(VECTOR_COMPY));
+	codeGen->FP_PushRel32(GetVectorElement(fs, VECTOR_COMPZ));
+	codeGen->FP_PushRel32(GetVectorElement(ft, VECTOR_COMPX));
+	codeGen->FP_MulS();
+	codeGen->FP_SubS();
+	codeGen->FP_PullRel32(GetVectorElement(tempRegIndex, VECTOR_COMPY));
 
 	//Z
-	codeGen->FP_PushSingle(GetAccumulatorElement(VECTOR_COMPZ));
-	codeGen->FP_PushSingle(GetVectorElement(fs, VECTOR_COMPX));
-	codeGen->FP_PushSingle(GetVectorElement(ft, VECTOR_COMPY));
-	codeGen->FP_Mul();
-	codeGen->FP_Sub();
-	codeGen->FP_PullSingle(GetVectorElement(tempRegIndex, VECTOR_COMPZ));
+	codeGen->FP_PushRel32(GetAccumulatorElement(VECTOR_COMPZ));
+	codeGen->FP_PushRel32(GetVectorElement(fs, VECTOR_COMPX));
+	codeGen->FP_PushRel32(GetVectorElement(ft, VECTOR_COMPY));
+	codeGen->FP_MulS();
+	codeGen->FP_SubS();
+	codeGen->FP_PullRel32(GetVectorElement(tempRegIndex, VECTOR_COMPZ));
 
 	TestSZFlags(codeGen, dest, offsetof(CMIPS, m_State.nCOP2[tempRegIndex]), relativePipeTime, compileHints);
 
@@ -1538,11 +1596,11 @@ void VUShared::RSQRT(CMipsJitter* codeGen, uint8 nFs, uint8 nFsf, uint8 nFt, uin
 	}
 	codeGen->Else();
 	{
-		codeGen->FP_PushSingle(GetVectorElement(nFs, nFsf));
-		codeGen->FP_PushSingle(GetVectorElement(nFt, nFtf));
-		codeGen->FP_Rsqrt();
-		codeGen->FP_Mul();
-		codeGen->FP_PullSingle(destination);
+		codeGen->FP_PushRel32(GetVectorElement(nFs, nFsf));
+		codeGen->FP_PushRel32(GetVectorElement(nFt, nFtf));
+		codeGen->FP_RsqrtS();
+		codeGen->FP_MulS();
+		codeGen->FP_PullRel32(destination);
 
 		codeGen->PushCst(0);
 		codeGen->PullRel(offsetof(CMIPS, m_State.nCOP2DF));
@@ -1569,36 +1627,11 @@ void VUShared::SQbase(CMipsJitter* codeGen, uint8 dest, uint8 is)
 	}
 	else
 	{
-		codeGen->AddRef();
-
-		//Try some aligned 64-bit writes first
-		for(unsigned int i = 0; i < 3; i += 2)
-		{
-			uint8 mask = (0x03 << i);
-			if((dest & mask) == mask)
-			{
-				unsigned int element = (i ^ 0x03) - 1;
-				codeGen->PushTop();
-				codeGen->PushCst(element * 4);
-				codeGen->PushRel64(offsetof(CMIPS, m_State.nCOP2[is].nV[element]));
-				codeGen->Store64AtRefIdx(1);
-				dest &= ~mask;
-			}
-		}
-
-		//Write any remaining data
-		for(unsigned int i = 0; i < 4; i++)
-		{
-			if(VUShared::DestinationHasElement(static_cast<uint8>(dest), i))
-			{
-				codeGen->PushTop();      //Push dest memory ref
-				codeGen->PushCst(i * 4); //Push index
-				codeGen->PushRel(offsetof(CMIPS, m_State.nCOP2[is].nV[i]));
-				codeGen->StoreAtRefIdx(1);
-			}
-		}
-
-		codeGen->PullTop();
+		codeGen->MD_PushRel(offsetof(CMIPS, m_State.nCOP2[is]));
+		codeGen->MD_StoreAtRefIdxMasked(DestinationHasElement(dest, 0),
+		                                DestinationHasElement(dest, 1),
+		                                DestinationHasElement(dest, 2),
+		                                DestinationHasElement(dest, 3));
 	}
 }
 
@@ -1658,10 +1691,10 @@ void VUShared::SQRT(CMipsJitter* codeGen, uint8 nFt, uint8 nFtf, uint32 relative
 	size_t destination = g_pipeInfoQ.heldValue;
 	QueueInPipeline(g_pipeInfoQ, codeGen, LATENCY_SQRT, relativePipeTime);
 
-	codeGen->FP_PushSingle(GetVectorElement(nFt, nFtf));
-	codeGen->FP_Abs();
-	codeGen->FP_Sqrt();
-	codeGen->FP_PullSingle(destination);
+	codeGen->FP_PushRel32(GetVectorElement(nFt, nFtf));
+	codeGen->FP_AbsS();
+	codeGen->FP_SqrtS();
+	codeGen->FP_PullRel32(destination);
 
 	codeGen->PushCst(0);
 	codeGen->PullRel(offsetof(CMIPS, m_State.nCOP2DF));
@@ -1674,7 +1707,7 @@ void VUShared::SUB(CMipsJitter* codeGen, uint8 dest, uint8 fd, uint8 fs, uint8 f
 	{
 		//Source and target registers are the same, clear the vector instead of going through a SUB instruction
 		//SUB might generate NaNs instead of clearing the values like the game intended (ex.: Homura with 0xFFFF8000)
-		codeGen->MD_PushRelExpand(offsetof(CMIPS, m_State.nCOP2[0].nV0));
+		codeGen->MD_PushCstExpandW(0);
 		PullVector(codeGen, dest, fdOffset);
 		TestSZFlags(codeGen, dest, fdOffset, relativePipeTime, compileHints);
 	}
@@ -1788,11 +1821,19 @@ void VUShared::CheckPipeline(const REGISTER_PIPEINFO& pipeInfo, CMipsJitter* cod
 	codeGen->PushCst(relativePipeTime);
 	codeGen->Add();
 
-	codeGen->BeginIf(Jitter::CONDITION_LE);
-	{
-		FlushPipeline(pipeInfo, codeGen);
-	}
-	codeGen->EndIf();
+	codeGen->Cmp(Jitter::CONDITION_LE);
+	codeGen->PushTop();
+
+	//This needs to match behavior in FlushPipeline
+	codeGen->PushCst(0);
+	codeGen->PushRel(pipeInfo.counter);
+	codeGen->Select();
+	codeGen->PullRel(pipeInfo.counter);
+
+	codeGen->PushRel(pipeInfo.heldValue);
+	codeGen->PushRel(pipeInfo.value);
+	codeGen->Select();
+	codeGen->PullRel(pipeInfo.value);
 }
 
 void VUShared::QueueInPipeline(const REGISTER_PIPEINFO& pipeInfo, CMipsJitter* codeGen, uint32 latency, uint32 relativePipeTime)
@@ -1810,8 +1851,6 @@ void VUShared::CheckFlagPipeline(const FLAG_PIPEINFO& pipeInfo, CMipsJitter* cod
 	//the flag register every time (pipeTimes[i] <= (pipeTime + relativePipeTime))
 	for(unsigned int i = 0; i < FLAG_PIPELINE_SLOTS; i++)
 	{
-		codeGen->PushRelAddrRef(pipeInfo.timeArray);
-
 		//Compute index into array
 		codeGen->PushRel(pipeInfo.index);
 		codeGen->PushCst(i);
@@ -1819,32 +1858,32 @@ void VUShared::CheckFlagPipeline(const FLAG_PIPEINFO& pipeInfo, CMipsJitter* cod
 		codeGen->PushCst(FLAG_PIPELINE_SLOTS - 1);
 		codeGen->And();
 
-		codeGen->Shl(2);
-		codeGen->AddRef();
-		codeGen->LoadFromRef();
+		//Load value for true branch
+		//Loaded here to allow merging of Cmp and Select operations
+		codeGen->PushRelAddrRef(pipeInfo.valueArray);
+		codeGen->PushIdx(1);
+		codeGen->LoadFromRefIdx();
+
+		codeGen->PushRelAddrRef(pipeInfo.timeArray);
+		codeGen->PushIdx(2);
+		codeGen->LoadFromRefIdx();
 
 		codeGen->PushRel(offsetof(CMIPS, m_State.pipeTime));
 		codeGen->PushCst(relativePipeTime);
 		codeGen->Add();
 
-		codeGen->BeginIf(Jitter::CONDITION_LE);
-		{
-			codeGen->PushRelAddrRef(pipeInfo.valueArray);
+		codeGen->Cmp(Jitter::CONDITION_LE);
 
-			//Compute index into array
-			codeGen->PushRel(pipeInfo.index);
-			codeGen->PushCst(i);
-			codeGen->Add();
-			codeGen->PushCst(FLAG_PIPELINE_SLOTS - 1);
-			codeGen->And();
+		//True branch
+		codeGen->Swap();
 
-			codeGen->Shl(2);
-			codeGen->AddRef();
-			codeGen->LoadFromRef();
+		//False branch
+		codeGen->PushRel(pipeInfo.value);
 
-			codeGen->PullRel(pipeInfo.value);
-		}
-		codeGen->EndIf();
+		codeGen->Select();
+
+		codeGen->PullRel(pipeInfo.value);
+		codeGen->PullTop();
 	}
 }
 
