@@ -1,5 +1,5 @@
 #include <cstring>
-#include "../Log.h"
+#include "Log.h"
 #include "../states/RegisterStateFile.h"
 #include "../Ps2Const.h"
 #include "../TimeUtils.h"
@@ -42,12 +42,15 @@
 #define FUNCTION_CDREADDVDDUALINFO "CdReadDvdDualInfo"
 #define FUNCTION_CDLAYERSEARCHFILE "CdLayerSearchFile"
 
-constexpr uint64 COMMAND_READ_BASE_DELAY = TimeUtils::UsecsToCycles(PS2::IOP_CLOCK_OVER_FREQ, 100);
-constexpr uint64 COMMAND_READ_SECTOR_DELAY = TimeUtils::UsecsToCycles(PS2::IOP_CLOCK_OVER_FREQ, 500);
-constexpr uint64 COMMAND_SEEK_DELAY = TimeUtils::UsecsToCycles(PS2::IOP_CLOCK_OVER_FREQ, 100);
-
 using namespace Iop;
 
+//Some games that might be sensitive to those timings:
+// - Shadow Hearts: Covenant: SEEK_DELAY needs not to be too small or game might hang during transitions (menu and battle)
+// - Tekken 5.1 and Tekken 5.5: Can get bad textures in title screen if delays are not set properly.
+const uint64 CCdvdman::COMMAND_READ_BASE_DELAY = TimeUtils::UsecsToCycles(PS2::IOP_CLOCK_OVER_FREQ, 100);
+const uint64 CCdvdman::COMMAND_READ_SECTOR_DELAY = TimeUtils::UsecsToCycles(PS2::IOP_CLOCK_OVER_FREQ, 500);
+const uint64 CCdvdman::COMMAND_SEEK_DELAY = TimeUtils::UsecsToCycles(PS2::IOP_CLOCK_OVER_FREQ, 500);
+
 CCdvdman::CCdvdman(CIopBios& bios, uint8* ram)
     : m_bios(bios)
     , m_ram(ram)
