@@ -2,7 +2,7 @@
 #include <cstring>
 #include <vector>
 #include "Iop_Sio2.h"
-#include "../Log.h"
+#include "Log.h"
 #include "../states/RegisterStateFile.h"
 #include "../states/MemoryStateFile.h"
 
@@ -121,7 +121,6 @@ void CSio2::SaveState(Framework::CZipArchiveWriter& archive)
 
 void CSio2::SetButtonState(unsigned int padNumber, PS2::CControllerInfo::BUTTON button, bool pressed, uint8* ram)
 {
-	assert(padNumber < MAX_PADS);
 	if(padNumber >= MAX_PADS) return;
 
 	auto& padState = m_padState[padNumber];
@@ -668,6 +667,8 @@ void CSio2::DisassembleWrite(uint32 address, uint32 value)
 
 void CSio2::GetVibration(unsigned int padId, uint8& largeMotor, uint8& smallMotor)
 {
+	if(padId >= MAX_PADS) return;
+
 	auto& padState = m_padState[padId];
 	largeMotor = padState.largeMotor;
 	smallMotor = padState.smallMotor;
